import { SerializedError } from "@reduxjs/toolkit";
import { FetchBaseQueryError } from "@reduxjs/toolkit/dist/query";
import { ReactElement } from "react";
export declare type UseQueryResult<T> = {
    originalArgs?: unknown;
    data?: T;
    currentData?: T;
    error?: unknown;
    requestId?: string;
    endpointName?: string;
    startedTimeStamp?: number;
    fulfilledTimeStamp?: number;
    isUninitialized: boolean;
    isLoading: boolean;
    isFetching: boolean;
    isSuccess: boolean;
    isError: boolean;
    refetch: () => void;
};
export declare type MakeDataRequired<T extends readonly UseQueryResult<unknown>[]> = {
    [K in keyof T]: T[K] & {
        data: NonNullable<T[K]["data"]>;
    };
};
export declare type LoaderTransformFunction<QRU extends readonly UseQueryResult<unknown>[], R extends unknown> = (queries: MakeDataRequired<QRU>) => R;
export declare type OptionalGenericArg<T> = T extends never ? [] : [T];
export declare type CreateUseLoaderArgs<QRU extends readonly UseQueryResult<unknown>[], R extends unknown, A = never> = {
    queries: (...args: OptionalGenericArg<A>) => QRU;
    transform?: LoaderTransformFunction<QRU, R>;
};
export declare type UseLoader<A, R> = (...args: OptionalGenericArg<A>) => UseQueryResult<R>;
export declare type CreateLoaderType = <QRU extends readonly UseQueryResult<unknown>[], R extends unknown = MakeDataRequired<QRU>, A = never>(createLoaderArgs: CreateUseLoaderArgs<QRU, R, A>) => UseLoader<A, R>;
export declare type ComponentWithLoaderData<P extends Record<string, any>, R extends unknown> = (props: P, loaderData: R) => ReactElement;
export declare type InferLoaderData<T> = T extends Loader<any, infer X, any> ? X : T extends Loader<never, infer Y, any> ? Y : T extends Loader<any, infer Z, never> ? Z : never;
export declare type Component<P extends Record<string, any>> = (props: P) => ReactElement;
export declare type WithLoaderArgs<P extends unknown, R extends unknown, A = never> = Loader<P, R, A>;
export declare type WhileFetchingArgs<P extends unknown, R extends unknown> = {
    prepend?: (props: P, data?: R) => ReactElement;
    append?: (props: P, data?: R) => ReactElement;
};
export declare type CreateLoaderArgs<P extends unknown, QRU extends readonly UseQueryResult<unknown>[], R extends unknown = MakeDataRequired<QRU>, A = never> = Partial<CreateUseLoaderArgs<QRU, R, A>> & {
    queriesArg?: (props: P) => A;
    onLoading?: (props: P) => ReactElement;
    onError?: (props: P, error: FetchBaseQueryError | SerializedError, joinedQuery: UseQueryResult<undefined>) => ReactElement;
    /** @deprecated Using onFetching might result in loss of internal state. Use `whileFetching` instead, or pass the query to the component */
    onFetching?: (props: P, renderBody: () => ReactElement) => ReactElement;
    whileFetching?: WhileFetchingArgs<P, R>;
};
export declare type Loader<P extends unknown, R extends unknown, A = never> = {
    useLoader: UseLoader<A, R>;
    queriesArg?: (props: P) => A;
    onLoading?: (props: P) => ReactElement;
    onError?: (props: P, error: SerializedError | FetchBaseQueryError, joinedQuery: UseQueryResult<undefined>) => ReactElement;
    /** @deprecated Using onFetching might result in loss of internal state. Use `whileFetching` instead, or pass the query to the component */
    onFetching?: (props: P, renderBody: () => ReactElement) => ReactElement;
    whileFetching?: WhileFetchingArgs<P, R>;
    extend: <QRUb extends readonly UseQueryResult<unknown>[], Pb extends unknown = P, Rb extends unknown = QRUb extends unknown ? R : MakeDataRequired<QRUb>, Ab = A>(newLoader: Partial<CreateLoaderArgs<Pb, QRUb, Rb, Ab>>) => Loader<Pb, Rb, Ab>;
};
